// ####################################################################################
// Includes
// ####################################################################################

#include "stdafx.h"
#include "internal.h"

using namespace GregorView::Internal;


// ####################################################################################
// Init/Fin
// ####################################################################################

CGregorViewClassFactory::CGregorViewClassFactory(REFCLSID rclsid)
{
	::GetInstanceCounter().Increment();
	// save the class ID (so we can pass it on to any folder we create
	m_ClassId = rclsid;	
}

CGregorViewClassFactory::~CGregorViewClassFactory(void)
{
	::GetInstanceCounter().Decrement();
}


// ####################################################################################
// IUnknown
// ####################################################################################

STDMETHODIMP CGregorViewClassFactory::QueryInterface(REFIID riid, void ** ppObj)
{
	*ppObj = NULL;
	if(riid == IID_IUnknown || riid == IID_IClassFactory){
		*ppObj = (void*)(IClassFactory*)this;
		this->AddRef();
		return NOERROR;
	}else{
		return E_NOINTERFACE;
	}
}

STDMETHODIMP_(ULONG) CGregorViewClassFactory::AddRef(void)
{
	m_RefCounter.Increment();
	return m_RefCounter.GetCount();
}

STDMETHODIMP_(ULONG) CGregorViewClassFactory::Release(void)
{
	if(m_RefCounter.Decrement()){
		delete this;
		return 0;
	}else{
		return m_RefCounter.GetCount();
	}
}

		
// ####################################################################################
// IClassFactory
// ####################################################################################

STDMETHODIMP CGregorViewClassFactory::CreateInstance(IUnknown * pUnkOuter, REFIID riid, void ** ppObj)
{
	if(pUnkOuter != NULL){
		return CLASS_E_NOAGGREGATION;
	}else if(riid != IID_IUnknown && riid != IID_IPersistFolder && riid != IID_IShellFolder){
		return E_NOINTERFACE;
	}else{
		::GetTracer().TraceFormatted("CGregorViewClassFactory::CreateInstance riid <IID_IShellFolder>");
		// create
		CGregorViewShellFolder * pFolder = new CGregorViewShellFolder(m_ClassId);
		if(pFolder == NULL){
			return E_OUTOFMEMORY;
		}
		// set out parameter
		pFolder->AddRef();
		*ppObj = (void*)pFolder;
		// OK
		return S_OK;
	}
}


STDMETHODIMP CGregorViewClassFactory::LockServer(BOOL fLock)
{
	if(fLock){
		::GetLockCounter().Increment();
	}else{
		::GetLockCounter().Decrement();
	}
	return NOERROR;
}

