// ####################################################################################
// Includes
// ####################################################################################

#include "stdafx.h"
#include "stdio.h"
#include "internal.h"

using namespace GregorView::Internal;


// ####################################################################################
// Init/Fin
// ####################################################################################

CGregorViewEnumerator::CGregorViewEnumerator(GregorView::Managed::CFolderBase * pFolder)
{
	::GetInstanceCounter().Increment();
	m_pFolder = pFolder;
	m_NextIndex = 0;
	// identify (... now for testing, later for implementors)
	//STRRET str;
	//HRESULT hresSub = m_pFolder->GetDisplayNameOf(::GetTerminator(), SHGDN_FORPARSING, &str);
	//if(SUCCEEDED(hresSub) && str.uType == STRRET_CSTR){
	//	::GetTracer().TraceFormatted("CGregorViewEnumerator::CGregorViewEnumerator path <%s>", str.cStr);
	//}	
}

CGregorViewEnumerator::~CGregorViewEnumerator(void)
{
	::GetInstanceCounter().Decrement();
}


// ####################################################################################
// IUnknown
// ####################################################################################

STDMETHODIMP CGregorViewEnumerator::QueryInterface(REFIID riid, void ** ppObj)
{
	*ppObj = NULL;
	if(riid == IID_IUnknown){
		*ppObj = (void*)(IEnumIDList*)this; // (IShellIDList*) because we don't derive directly from IUnknown
		this->AddRef();
		return NOERROR;
	}else if(riid == IID_IEnumIDList){
		::GetTracer().Trace("CGregorViewEnumerator::QueryInterface riid <IID_IEnumIDList>");
		*ppObj = (void*)(IEnumIDList*)this;
		this->AddRef();
		return NOERROR;
	}else{
		return E_NOINTERFACE;
	}
}

STDMETHODIMP_(ULONG) CGregorViewEnumerator::AddRef(void)
{
	m_RefCounter.Increment();
	return m_RefCounter.GetCount();
}

STDMETHODIMP_(ULONG) CGregorViewEnumerator::Release(void)
{
	if(m_RefCounter.Decrement()){
		delete this;
		return 0;
	}else{
		return m_RefCounter.GetCount();
	}
}


// ####################################################################################
// IEnumIDList
// ####################################################################################

STDMETHODIMP CGregorViewEnumerator::Clone(IEnumIDList ** ppEnum)
{
	*ppEnum = NULL;
	CGregorViewEnumerator * pClone = new CGregorViewEnumerator(m_pFolder);
	if(pClone == NULL){
		return E_OUTOFMEMORY;
	}
	pClone->m_NextIndex = this->m_NextIndex;
	*ppEnum = pClone;
	return NOERROR;
}

STDMETHODIMP CGregorViewEnumerator::Next(ULONG celt, ITEMIDLIST ** ppOut, ULONG * pCountFetched)
{
	HRESULT hres = NOERROR; // exactly the number of requested elements are returned
	// walk: limit to requested count && available items
	for(ULONG u = 0; u < celt; u++){
		if(m_NextIndex < m_pFolder->SubFolders->Count){
			// save index, advance
			int index = m_NextIndex;
			m_NextIndex++;
			// get sub folder
			GregorView::Managed::CFolderBase * pSubFolder = m_pFolder->SubFolders->get_Element(index);
			if(pSubFolder == NULL){
				hres = E_UNEXPECTED;
				break;
			}
			// prepare id properties
			const char * pszName = SysStringToCString(pSubFolder->DisplayName);
			if(pszName == NULL){
				hres = E_OUTOFMEMORY;
				break;
			}
			DWORD dwAttribs = (DWORD) pSubFolder->ShellAttributes;
			// create id
			CGregorViewId * pID = new CGregorViewId(dwAttribs, pszName);
			if(pID == NULL){
				hres = E_OUTOFMEMORY;
				break;
			}			
			// put into array
			ppOut[u] = pID->AsItemID();			
			// update out count
			if(pCountFetched != NULL){	// can be NULL if celt is 1
				(*pCountFetched)++;
			}
		}else{
			hres = S_FALSE; // end of sequence, out count is smaller than requested count
			break;
		}
	}
	// return
	return hres;
}

STDMETHODIMP CGregorViewEnumerator::Reset(void)
{
	m_NextIndex = 0;
	return NOERROR;
}

STDMETHODIMP CGregorViewEnumerator::Skip(ULONG celt)
{
	m_NextIndex += (int)celt;
	return NOERROR;
}
