// ####################################################################################
// Includes
// ####################################################################################

#include "stdafx.h"
#include "internal.h"

#using <mscorlib.dll>

using namespace System;
using namespace System::Reflection;
using namespace System::Runtime::InteropServices;
using namespace GregorView::Internal;
using namespace GregorView::Managed;


// ####################################################################################
// Init/Fin
// ####################################################################################

CGregorViewExtensionPoint::CGregorViewExtensionPoint(REFCLSID rclsid, const char * pszClassId, const char * pszManagedDllPath)
{
	// save class ID
	m_ClassId = rclsid;
	strncpy(m_szClassId, pszClassId, sizeof(m_szClassId));
	m_szClassId[sizeof(m_szClassId) - 1] = '\0';
	// reset NSE root pidl
	m_pIDListNseRoot = NULL;
	// save managed DLL path
	strncpy(m_szManagedDllPath, pszManagedDllPath, sizeof(m_szManagedDllPath));
	m_szManagedDllPath[sizeof(m_szManagedDllPath) - 1] = '\0';
	// reset connector reference
	m_pConnector = NULL;
}

CGregorViewExtensionPoint::~CGregorViewExtensionPoint(void)
{
	return;
}


// ####################################################################################
// Properties
// ####################################################################################

const CLSID &  CGregorViewExtensionPoint::GetClassId(void)
{
	return m_ClassId;
}

const char * CGregorViewExtensionPoint::GetClassIdAsString(void)
{
	return m_szClassId;
}

const ITEMIDLIST * CGregorViewExtensionPoint::GetIDList(void)
{
	return m_pIDListNseRoot;
}


// ####################################################################################
// ID List Methods
// ####################################################################################

bool CGregorViewExtensionPoint::SaveNseRoot(const ITEMIDLIST * pcidl)
{
	if(m_pIDListNseRoot == NULL){
		m_pIDListNseRoot = CloneIDList(pcidl);
		GetTracer().Trace("CGregorViewExtensionPoint::SaveNseRoot saved ID list of NSE root folder"); 
		return true;
	}else{
		// the first ID list saved (per class ID) is the root; but this method may
		// explicitly be called for any folder; also, the NSE root pidl might just
		// be passed to IPersistFolder::Initialize multiple times ... (not clear)
		return false;
	}
}

const ITEMIDLIST * CGregorViewExtensionPoint::GetNseRoot(void)
{
	return m_pIDListNseRoot;
}

const ITEMIDLIST * CGregorViewExtensionPoint::SkipNseRoot(const ITEMIDLIST * pcidl)
{
	return GregorView::Internal::Skip(pcidl, m_pIDListNseRoot);
}


// ####################################################################################
// Connection Methods
// ####################################################################################

CConnector * CGregorViewExtensionPoint::GetConnector(void)
{
	if(m_pConnector == NULL){
		try{
			// convert assembly path to System::String
			String * ps = CStringToSysString(m_szManagedDllPath);
			// load assembly
			Assembly * pAssembly = Assembly::LoadFrom(ps);
			// walk types, find connector
			Type * pTypes[] = pAssembly->GetTypes();
			Type * pConnectorBase = __typeof(CConnector);
			for(int i = 0; i < pTypes->Length; i++){
				Type * pType = pTypes[0];
				if(pType->IsSubclassOf(pConnectorBase)){
					// instantiate
					Object * pObj = Activator::CreateInstance(pType);
					if(pObj != NULL){
						m_pConnector = __try_cast<CConnector*>(pObj);
						break;
					}
				}
			}
		}catch(Exception * pExc){
			(void)pExc;
		}
	}
	// return
	return m_pConnector;
}

CFolderBase * CGregorViewExtensionPoint::GetFolder(CGregorViewShellFolder * pShellFolder)
{
	// get absolute ID list
	const ITEMIDLIST * pcidlAbsolute = pShellFolder->GetIDList();
	if(pcidlAbsolute == NULL){
		return NULL;
	}
	// skip NSE root (get relative, possibly multi-level ID list)
	const ITEMIDLIST * pcidlRelative = this->SkipNseRoot(pcidlAbsolute);
	if(pcidlRelative == NULL){
		// even after the skipping of the NSE root, there should still be a terminator
		return NULL;
	}
	// get the fucking connector
	CConnector * pConnector = this->GetConnector();
	if(pConnector == NULL){
		return NULL;
	}	
	// get the NSE root folder anyway
	CFolderBase * pRootFolder = pConnector->GetRootFolder();
	if(pRootFolder == NULL){
		return NULL;
	}
	// if the ID list ends right after the NSE root ID list, we're talking about the NSE root folder
	if(IsTerminator(pcidlRelative)){
		return pRootFolder;
	}
	// walk the relative ID list, asking the managed DLL to enum folders at every level
	CFolderBase * pFolder = pRootFolder;
	while(pcidlRelative != NULL && false == IsTerminator(pcidlRelative)){
		// reinterpret
		CGregorViewId * pID = AsGregorViewId(pcidlRelative);
		if(pID == NULL){
			return NULL;
		}
		// get the name
		String * psName = CStringToSysString(pID->GetName());
		if(psName == NULL){
			return NULL;
		}
		// ask the folder for a sub folder of that name (this will trigger a collection fill as needed)
		pFolder = pFolder->SubFolders->get_Element(psName);
		if(pFolder == NULL){
			return NULL; // no match
		}
		// next single-level ID
		pcidlRelative = Next(pcidlRelative);
	}
	
	// return match, if any
	return pFolder;
}

