// ####################################################################################
// Includes
// ####################################################################################

#include "stdafx.h"
#include "internal.h"


// ####################################################################################
// Init/Fin
// ####################################################################################

CGregorViewFolder::CGregorViewFolder(void)
{
	::GetInstanceCounter().Increment();
	m_pIDList = NULL;
}

CGregorViewFolder::~CGregorViewFolder(void)
{
	if(m_pIDList != NULL){
		::GetAllocator().Free((void*)m_pIDList);
	}
	::GetInstanceCounter().Decrement();
}


// ####################################################################################
// IUnknown
// ####################################################################################

STDMETHODIMP CGregorViewFolder::QueryInterface(REFIID riid, void ** ppObj)
{
	*ppObj = NULL;
	if(riid == IID_IUnknown){
		*ppObj = (void*)(IShellFolder*)this; // (IShellFolder*) because we don't derive directly from IUnknown
		this->AddRef();
		return NOERROR;
	}else if(riid == IID_IPersistFolder){
		::GetTracer().Trace("CGregorViewFolder::QueryInterface riid <IID_IPersistFolder>");
		*ppObj = (void*)(IPersistFolder*)this;
		this->AddRef();
		return NOERROR;
	}else if(riid == IID_IShellFolder){
		::GetTracer().Trace("CGregorViewFolder::QueryInterface riid <IID_IShellFolder>");
		*ppObj = (void*)(IShellFolder*)this;
		this->AddRef();
		return NOERROR;		
	}else{
		return E_NOINTERFACE;
	}
}

STDMETHODIMP_(ULONG) CGregorViewFolder::AddRef(void)
{
	m_RefCounter.Increment();
	return m_RefCounter.GetCount();
}

STDMETHODIMP_(ULONG) CGregorViewFolder::Release(void)
{
	if(m_RefCounter.Decrement()){
		delete this;
		return 0;
	}else{
		return m_RefCounter.GetCount();
	}
}


// ####################################################################################
// IShellFolder
// ####################################################################################

// ... implement IShellFolder methods

STDMETHODIMP CGregorViewFolder::CreateViewObject( 
										HWND hwndOwner,  
										REFIID riid, 
										LPVOID FAR* ppvObj) 
{
	::GetTracer().Trace("CGregorViewFolder::CreateViewObject");
	// check
	if(riid != IID_IShellView){
		return E_NOINTERFACE;
	}
	// create
	CGregorViewFolderView * pFolderView = new CGregorViewFolderView(hwndOwner);
	if(pFolderView == NULL){
		return E_OUTOFMEMORY;
	}
	// set out parameter
	pFolderView->AddRef();
	*ppvObj = (void*)pFolderView;
	// OK
	return NOERROR;
} 
 
STDMETHODIMP CGregorViewFolder::EnumObjects( 
										HWND hwndOwner,  
										DWORD grfFlags, 
										LPENUMIDLIST * ppEnumIDList)
{ 
	::GetTracer().Trace("CGregorViewFolder::EnumObjects");
	// reset out parameter
	*ppEnumIDList = NULL;
	// create walker
	CGregorViewEnumerator * pWalker = new CGregorViewEnumerator(this); // ... process flags
	if(pWalker == NULL){
		return E_OUTOFMEMORY;
	}
	// set out parameter
	pWalker->AddRef();
	*ppEnumIDList = (IEnumIDList*)pWalker;
	// OK
	return NOERROR;
}
 
STDMETHODIMP CGregorViewFolder::BindToObject( 
										LPCITEMIDLIST pidl,
										LPBC pbc,
										REFIID riid,
										LPVOID FAR* ppvObj)
{
	// reset out parameter
	*ppvObj = NULL;
	// we support only IShellFolder for now
	if(riid == IID_IShellFolder){
		::GetTracer().TraceFormatted("CGregorViewFolder::BindToObject riid <IID_IShellFolder>");
		// create folder
		CGregorViewFolder * pFolder = new CGregorViewFolder();
		if(pFolder == NULL){
			return E_OUTOFMEMORY;
		}
		// save pidl (can be multi-level, but is always relative to this folder)
		pFolder->m_pIDList = ::ConcatIDLists(m_pIDList, pidl);
		// pidl info
		::GetTracer().TraceFormatted("CGregorViewFolder::BindToObject this <0x%x>", this);
		::GetTracer().TraceFormatted("CGregorViewFolder::BindToObject parent abs pidl %s", ::GetIDListInfoString(m_pIDList));
		::GetTracer().TraceFormatted("CGregorViewFolder::BindToObject child  rel pidl %s", ::GetIDListInfoString(pidl));
		::GetTracer().TraceFormatted("CGregorViewFolder::BindToObject child  abs pidl %s", ::GetIDListInfoString(pFolder->m_pIDList));
		// return
		pFolder->AddRef();
		*ppvObj = (void*)pFolder;
		return NOERROR;
	}else{
		return E_NOINTERFACE;
	}
}
 
STDMETHODIMP CGregorViewFolder::BindToStorage(
										LPCITEMIDLIST pidl,  
										LPBC pbc, 
										REFIID riid,  
   LPVOID FAR* ppvObj)
{ 
	::GetTracer().Trace("CGregorViewFolder::BindToStorage"); 
	return E_NOTIMPL;
} 

STDMETHODIMP CGregorViewFolder::CompareIDs(
										LPARAM lParam,
										LPCITEMIDLIST pidl1,
										LPCITEMIDLIST pidl2)
{
	::GetTracer().Trace("CGregorViewFolder::CompareIDs");
	// reinterpret some
	CGregorViewId * pID1 = CGregorViewId::FromItemID(pidl1);
	CGregorViewId * pID2 = CGregorViewId::FromItemID(pidl2);
	// delegate comparison to ID class
	short sResult = pID1->CompareTo(pID2);
	// return the result as an HRESULT
	HRESULT hres = MAKE_HRESULT(SEVERITY_SUCCESS, 0, sResult);
	return hres;
}

STDMETHODIMP CGregorViewFolder::ParseDisplayName(
										HWND hwndOwner,
										LPBC pbc,
										LPOLESTR lpszDisplayName,
										ULONG FAR* pchEaten,
										LPITEMIDLIST * ppidl,
										ULONG *pdwAttributes)
{ 
	::GetTracer().Trace("CGregorViewFolder::ParseDisplayName");
	// convert to ANSI
	char sz[MAX_PATH];
	if(0 == ::WideCharToMultiByte(CP_ACP, 0, lpszDisplayName, -1, sz, sizeof(sz), NULL, NULL)){
		return E_UNEXPECTED;
	}
	::GetTracer().TraceFormatted("CGregorViewFolder::ParseDisplayName path <%s>", sz);
	// ... validate display name (is relative, can be multi-level, can be called on any of our folders)
	// parse
	const ITEMIDLIST * pcidl = NULL;
	char * psz = strtok(sz, "\\");
	while(psz != NULL){
		// create single-level item ID
		CGregorViewId * pID = new CGregorViewId(SFGAO_FOLDER | SFGAO_HASSUBFOLDER, psz);
		if(pID == NULL){
			return E_OUTOFMEMORY;
		}
		// concat?
		if(pcidl == NULL){
			pcidl = pID->AsItemID();
		}else{
			const ITEMIDLIST * pcidlNew = ::ConcatIDLists(pcidl, pID->AsItemID());
			if(pcidlNew == NULL){
				return E_OUTOFMEMORY;
			}
			::GetAllocator().Free((ITEMIDLIST*)pcidl);
			delete pID;
			pcidl = pcidlNew;
		}
		// eat away
		if(pchEaten != NULL){
			(*pchEaten) += (ULONG)strlen(psz);
		}
		// next
		psz = strtok(NULL, "\\");
	}
	*ppidl = const_cast<ITEMIDLIST*>(pcidl);
	// ... set attributes to nada if client can't be arsed to make another call
	if(pdwAttributes != NULL){
		 *pdwAttributes = 0;
	}
	// OK
	return NOERROR;
}

STDMETHODIMP CGregorViewFolder::GetDisplayNameOf( 
										LPCITEMIDLIST pidl,  
										DWORD dwFlags,  
										LPSTRRET lpName) 
{
	::GetTracer().Trace("CGregorViewFolder::GetDisplayNameOf");
	// so
	if((dwFlags & SHGDN_INFOLDER) == SHGDN_INFOLDER){
		::GetTracer().Trace("CGregorViewFolder::GetDisplayNameOf SHGDN_INFOLDER");
		// relative display name (under folder)
		CGregorViewId * pID = CGregorViewId::FromItemID(pidl);
		pID->GetName(lpName);
	}else{
		::GetTracer().Trace("CGregorViewFolder::GetDisplayNameOf SHGDN_NORMAL");
		// ... what about virtual folder class IDs above the NSE root?
		// absolute display name (for editing, address bar, parsing)
		char sz[MAX_PATH];
		if((dwFlags & SHGDN_FORPARSING) == SHGDN_FORPARSING){
			strcpy(sz, CLSID_GREGORVIEW_DISPLAYNAME);
		}else{
			strcpy(sz, GREGORVIEW_NAME);
		}
		// concat names of parent folders (ID list must be relative to the NSE root folder)
		const ITEMIDLIST * pcidlThisUnderNseRoot = ::SkipNseRoot(m_pIDList);
		if(pcidlThisUnderNseRoot == NULL){
			return E_UNEXPECTED;
		}
		::ConcatPathOfNamesTo(pcidlThisUnderNseRoot, sz, sizeof(sz));
		// add names of items (ID list can be multi level if SHGDN_FORPARSING is set, but ID list is relative to this folder)
		::ConcatPathOfNamesTo(pidl, sz, sizeof(sz));
		// OK
		lpName->uType = STRRET_CSTR;
		strncpy(lpName->cStr, sz, MAX_PATH);
		lpName->cStr[MAX_PATH - 1] = '\0';
	}
	return NOERROR;
}

STDMETHODIMP CGregorViewFolder::SetNameOf( 
										HWND hwndOwner,  
										LPCITEMIDLIST pidl, 
										LPCOLESTR lpszName,  
										DWORD dwReserved, 
										LPITEMIDLIST FAR* ppidlOut) 
{ 
	::GetTracer().Trace("CGregorViewFolder::SetNameOf");
	return E_NOTIMPL;
}

STDMETHODIMP CGregorViewFolder::GetAttributesOf( 
										UINT cidl,  
										LPCITEMIDLIST FAR* apidl, 
										ULONG FAR* rgfInOut) 
{ 
	::GetTracer().Trace("CGregorViewFolder::GetAttributesOf");
	// walk the request IDs
	ULONG uCommonAttribs = 0xFFFFFFFF; 
	for(UINT u = 0; u < cidl; u++){
		// reinterpret
		CGregorViewId * pID = CGregorViewId::FromItemID(apidl[u]);
		// get attributes
		ULONG uAttribs = pID->GetAttribs();
		// filter out attribs not common to all IDs in the array
		uCommonAttribs = (uCommonAttribs & uAttribs);
	}
	// only one single ULONG is there, which contains only common flags
	*rgfInOut = uCommonAttribs;
	// OK
	return NOERROR;
}

STDMETHODIMP CGregorViewFolder::GetUIObjectOf(
										HWND hwndOwner,  
										UINT cidl,  
										LPCITEMIDLIST FAR* apidl,  
										REFIID riid,  
										UINT FAR* prgfInOut,  
										LPVOID FAR* ppvObj) 
{
	::GetTracer().Trace("CGregorViewFolder::GetUIObjectOf");
	// reset
	*ppvObj = NULL;
	// switch
	//if(riid == IID_IExtractIcon){ // cidl can only be 1
	//	::GetTracer().TraceFormatted("CGregorViewFolder::GetUIObjectOf riid <IID_IExtractIcon> cidl <%d>", cidl);
	//	CGregorViewIconExtractor * pExtractor = new CGregorViewIconExtractor();
	//	pExtractor->AddRef();
	//	*ppvObj = (IExtractIcon*)pExtractor;
	//	return NOERROR;
	//}else{
		return E_NOINTERFACE;
	//}
}


// ####################################################################################
// IPersistFolder
// ####################################################################################

STDMETHODIMP CGregorViewFolder::GetClassID(CLSID * pClassID)
{
	::GetTracer().Trace("CGregorViewFolder::GetClassID");
	*pClassID = CLSID_GREGORVIEW;
	return S_OK;
}

STDMETHODIMP CGregorViewFolder::Initialize(const ITEMIDLIST * pidl)
{
	::GetTracer().TraceFormatted("CGregorViewFolder::Initialize this <0x%x> pidl %s", this, ::GetIDListInfoString(pidl));
	// keep a clone of the ID list leading to the NSE's root folder
	::SaveNseRoot(pidl);
	// keep a clone of the ID list
	m_pIDList = ::CloneIDList(pidl);
	::GetTracer().TraceFormatted("CGregorViewFolder::Initialize this <0x%x> m_pIDList %s", this, ::GetIDListInfoString(m_pIDList));
	// OK
	return NOERROR;
}


// ####################################################################################
// Other
// ####################################################################################

const ITEMIDLIST * CGregorViewFolder::GetIDList(void)
{
	return m_pIDList;
}
