// ####################################################################################
// Includes
// ####################################################################################

#include "stdafx.h"
#include "internal.h"

using namespace GregorView::Internal;


// ####################################################################################
// Init/Fin
// ####################################################################################

CGregorViewIconExtractor::CGregorViewIconExtractor(void)
{
	::GetInstanceCounter().Increment();
}

CGregorViewIconExtractor::~CGregorViewIconExtractor(void)
{
	::GetInstanceCounter().Decrement();
}


// ####################################################################################
// IUnknown
// ####################################################################################

STDMETHODIMP CGregorViewIconExtractor::QueryInterface(REFIID riid, void ** ppObj)
{
	*ppObj = NULL;
	if(riid == IID_IUnknown){
		*ppObj = (void*)(IExtractIcon*)this; // (IExtractIcon*) because we don't derive directly from IUnknown
		this->AddRef();
		return NOERROR;
	}else if(riid == IID_IExtractIcon){
		::GetTracer().Trace("CGregorViewShellFolder::QueryInterface riid <IID_IExtractIcon>");
		*ppObj = (void*)(IExtractIcon*)this;
		this->AddRef();
		return NOERROR;
	}else{
		return E_NOINTERFACE;
	}
}

STDMETHODIMP_(ULONG) CGregorViewIconExtractor::AddRef(void)
{
	m_RefCounter.Increment();
	return m_RefCounter.GetCount();
}

STDMETHODIMP_(ULONG) CGregorViewIconExtractor::Release(void)
{
	if(m_RefCounter.Decrement()){
		delete this;
		return 0;
	}else{
		return m_RefCounter.GetCount();
	}
}


// ####################################################################################
// IExtractIcon
// ####################################################################################

HRESULT CGregorViewIconExtractor::GetIconLocation(UINT uFlags, LPTSTR szIconFile, UINT cchMax, int *piIndex, UINT *pwFlags)
{
	::GetTracer().TraceFormatted("CGregorViewIconExtractor::GetIconLocation");
	*pwFlags = GIL_NOTFILENAME | GIL_PERCLASS; // ... parameterize icons
	return S_OK;
}

HRESULT CGregorViewIconExtractor::Extract(LPCTSTR pszFile, UINT nIconIndex, HICON *phiconLarge, HICON *phiconSmall, UINT nIconSize)
{
	HRESULT hres = NOERROR;
	::SetLastError(0);
	HANDLE h = ::LoadImage((HINSTANCE)::GetLibraryHandle(), MAKEINTRESOURCE(IDI_ICON1), IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR | LR_DEFAULTSIZE);
	int err = GetLastError();
	::GetTracer().TraceFormatted("CGregorViewIconExtractor::Extract h <0x%x>", h);
	if(h == NULL){
		::GetTracer().TraceFormatted("CGregorViewIconExtractor::Extract err <%d>", err);
		hres = E_UNEXPECTED;
	}else{
		*phiconLarge = (HICON)h; // ... really support large icons
		*phiconSmall = (HICON)h;
	}
	return hres;
}

