// ####################################################################################
// Includes
// ####################################################################################

#include "stdafx.h"
#include "internal.h"

using namespace GregorView::Internal;


// ####################################################################################
// Allocation/Deallocation
// ####################################################################################

void * CGregorViewId::operator new(size_t size)
{
	size += TERM_SIZE; // make room for terminator
	CGregorViewId * pID = reinterpret_cast<CGregorViewId*>(::GetAllocator().Allocate(size));
	return pID;
}

void CGregorViewId::operator delete(void * p)
{
	::GetAllocator().Free(p);
}



// ####################################################################################
// Static Functions
// ####################################################################################

CGregorViewId * CGregorViewId::FromItemID(const ITEMIDLIST * pcidl)
{
	ITEMIDLIST * pidl = const_cast<ITEMIDLIST*>(pcidl);
	return reinterpret_cast<CGregorViewId*>(pidl);
}


// ####################################################################################
// Init/Fin
// ####################################################################################

CGregorViewId::CGregorViewId(DWORD dwAttribs, const char * pszName)
{
	// save size (our type does not include the terminator)
	this->cb = sizeof(*this);
	// set properties
	this->wReserved = 0;
	this->dwReserved = 0;
	this->dwAttribs = dwAttribs;
	const int len = sizeof(this->szName) - 1;
	strncpy(this->szName, pszName, len);
	this->szName[len] = '\0';
	// make sure there's terminator (we've been allocated TERM_SIZE extra bytes)
	memset(this + sizeof(*this), 0, TERM_SIZE);
}


// ####################################################################################
// Methods
// ####################################################################################

DWORD CGregorViewId::GetAttribs(void)
{
	return this->dwAttribs;
}

const char * CGregorViewId::GetName(void)
{
	return this->szName;
}

void CGregorViewId::GetName(STRRET * pStr)
{
	pStr->uType = STRRET_CSTR;
	strncpy(pStr->cStr, this->szName, MAX_PATH);
	pStr->cStr[MAX_PATH] = '\0';
}

void CGregorViewId::GetName(STRRET * pStr, const char * pszParentNames)
{
	pStr->uType = STRRET_CSTR;
	strncpy(pStr->cStr, pszParentNames, MAX_PATH);
	strncat(pStr->cStr, "\\", MAX_PATH - strlen(pStr->cStr) - 1);
	strncat(pStr->cStr, this->szName, MAX_PATH - strlen(pStr->cStr) - 1);
	pStr->cStr[MAX_PATH - 1] = '\0';
}

void CGregorViewId::ConcatNameTo(char * psz, size_t bufferSize)
{
	strncat(psz, "\\", bufferSize - strlen(psz));
	strncat(psz, this->szName, bufferSize - strlen(psz));
	psz[bufferSize - 1] = '\0';
}

signed short CGregorViewId::CompareTo(CGregorViewId * that)
{
	if((this->dwAttribs & SFGAO_FOLDER) == (that->dwAttribs & SFGAO_FOLDER)){
		return (signed short) strncmp(this->szName, that->szName, sizeof(this->szName));
	}else{
		if((this->dwAttribs & SFGAO_FOLDER) == SFGAO_FOLDER){
			return (signed short) 1;
		}else{
			return (signed short) -1;
		}
	} 
}

ITEMIDLIST * CGregorViewId::AsItemID(void)
{
	return reinterpret_cast<ITEMIDLIST*>(this);
}