// ####################################################################################
// Includes
// ####################################################################################

#include "stdafx.h"
#include "internal.h"

using namespace GregorView::Internal;


// ####################################################################################
// Static Data
// ####################################################################################

static HANDLE m_hModule;
static CCounter m_InstanceCounter;
static CCounter m_LockCounter;
static CAllocator m_Allocator;
static CTracer m_Tracer;
static NGregorViewExtensionPoints m_ExtensionPoints;


// ####################################################################################
// DllMain
// ####################################################################################

BOOL APIENTRY DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			m_hModule = hModule;
			break;
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}

	// ... remove these questionable calls from DllMain!
	switch(ul_reason_for_call){
		case DLL_PROCESS_ATTACH:
		case DLL_PROCESS_DETACH:
			//::Beep(ul_reason_for_call * 500 + 100, 500);
			break;
	}
	::GetTracer().Trace("::DllMain reason", ul_reason_for_call); 

    return TRUE;
}


// ####################################################################################
// COM Exports
// ####################################################################################

STDAPI DllCanUnloadNow()
{
	// we can unload if client process has neither objects nor locks in use
	if(m_InstanceCounter.IsZero() && m_LockCounter.IsZero()){
		::GetTracer().Trace("::DllCanUnloadNow <S_OK>");
		return S_OK; // no fancy statement cramming here: S_OK != TRUE !!!
	}else{
		::GetTracer().Trace("::DllCanUnloadNow <S_FALSE>");
		//::Beep(2500, 250); ::Beep(2500, 250); ::Beep(2500, 250);
		return S_FALSE;
	}
} // DllCanUnloadNow

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID * ppv)
{
	// hi
	::GetTracer().Trace("::DllGetClassObject");
	// reset out parameter
	*ppv = NULL;
	// valicate requested interface
	if(riid != IID_IClassFactory
	|| ppv == NULL){
		return E_INVALIDARG;
	}
	// make sure extension point exists or can be created (this will verify the class ID)
	CGregorViewExtensionPoint * pExPoint = GetExtensionPoints().GetOrCreate(rclsid);
	if(pExPoint == NULL){
		return E_INVALIDARG;
	}
	// create class factory
	CGregorViewClassFactory * pFactory = new CGregorViewClassFactory(rclsid);
	if(pFactory == NULL){
		return E_OUTOFMEMORY;
	}
	// set out parameter
	pFactory->AddRef();
	*ppv = (void*)pFactory;
	// bye
	::GetTracer().Trace("::DllGetClassObject success");
	// return
	return S_OK;

} // DllGetClassObject


// ####################################################################################
// Internal Functions
// ####################################################################################

HANDLE GregorView::Internal::GetLibraryHandle(void)
{
	return m_hModule;
}

CCounter & GregorView::Internal::GetInstanceCounter(void)
{
	return m_InstanceCounter;
}

CCounter & GregorView::Internal::GetLockCounter(void)
{
	return m_LockCounter;
}

CAllocator & GregorView::Internal::GetAllocator(void)
{
	return m_Allocator;
}

CTracer & GregorView::Internal::GetTracer(void)
{
	return m_Tracer;
}

NGregorViewExtensionPoints & GregorView::Internal::GetExtensionPoints(void)
{
	return m_ExtensionPoints;
}

bool GregorView::Internal::GetManagedImplementationPath(const char * pszClassId, char * pszPath, size_t sizePath)
{
	// open the key
	char szSubKey[1000];
	strcpy(szSubKey, "CLSID\\");
	strcat(szSubKey, pszClassId);
	strcat(szSubKey, "\\ManagedImplementation");
	HKEY hKeyResult = NULL;
	if(ERROR_SUCCESS != RegOpenKeyEx(HKEY_CLASSES_ROOT, szSubKey, 0, KEY_ALL_ACCESS, &hKeyResult)){
		return false;
	}
	// read the value
	char szValue[MAX_PATH];
	size_t size = sizeof(szValue);
	if(ERROR_SUCCESS == RegQueryValueEx(hKeyResult, NULL /* use default value */, NULL, NULL, (LPBYTE)szValue, (LPDWORD)&size)){
		strncpy(pszPath, szValue, sizePath);
		pszPath[sizePath - 1] = '\0';
	}
	// close the key
	RegCloseKey(hKeyResult);
	// return success
	return true;
}
