#pragma once

#using <mscorlib.dll>

using namespace System;
using namespace System::Collections;

namespace GregorView { namespace Managed {

	public __gc class CConnector;
	public __gc class CItemBase;
	public __gc class CFolderBase;
	public __gc class NFolderCollection;

	public __value enum ShellAttributes
	{
		CanCopy           = 0x1,			 /* Objects can be copied == DROPEFFEC_MOVE */
		CanMove           = 0x2,			 /* Objects can be moved == DROPEFFECT_MOVE */   
		CanLink           = 0x4,			 /* Objects can be linked == DROPEFFECT_LINK */ 
		Storage           = 0x00000008L,     /* supports BindToObject(IID_IStorage) */
		CanRename         = 0x00000010L,     /* Objects can be renamed */
		CanDelete         = 0x00000020L,     /* Objects can be deleted */
		HasPropsheet      = 0x00000040L,     /* Objects have property sheets */
		DropTarget        = 0x00000100L,     /* Objects are drop target */
		CapabilityMask    = 0x00000177L,
		Encrypted         = 0x00002000L,     /* object is encrypted (use alt color) */
		IsSlow            = 0x00004000L,     /* 'slow' object */
		Ghosted           = 0x00008000L,     /* ghosted icon */
		Link              = 0x00010000L,     /* Shortcut (link) */
		Share             = 0x00020000L,     /* shared */
		ReadOnly          = 0x00040000L,     /* read-only */
		Hidden            = 0x00080000L,     /* hidden object */
		DisplayAtrrMask   = 0x000FC000L,
		FileSysAncestor   = 0x10000000L,     /* may contain children with SFGAO_FILESYSTEM */
		Folder            = 0x20000000L,     /* support BindToObject(IID_IShellFolder) */
		FileSystem        = 0x40000000L,     /* is a win32 file system object (file/folder/root) */
		HasSubFolder      = 0x80000000L,     /* may contain children with SFGAO_FOLDER */
		ContentsMask      = 0x80000000L,
		Validate          = 0x01000000L,     /* invalidate cached information */
		Removable         = 0x02000000L,     /* is this removeable media? */
		Compressed        = 0x04000000L,     /* Object is compressed (use alt color) */
		Browsable         = 0x08000000L,     /* supports IShellFolder, but only implements CreateViewObject() (non-folder view) */
		NonEnumerated     = 0x00100000L,     /* is a non-enumerated object */
		NewContent        = 0x00200000L,     /* should show bold in explorer tree */
		CanMoniker        = 0x00400000L,     /* defunct */
		HasStorage        = 0x00400000L,     /* defunct */
		Stream            = 0x00400000L,     /* supports BindToObject(IID_IStream) */
		StorageAncestor   = 0x00800000L,     /* may contain children with SFGAO_STORAGE or SFGAO_STREAM */
		StorageCapMask    = 0x70C50008L      /* for determining storage capabilities, ie for open/save semantics */
	};

	public __abstract __gc class CConnector : public IDisposable
	{
		public:
			// init/fin
			CConnector(void);
			virtual void Dispose(void);
			// connection methods
			virtual CFolderBase * GetRootFolder(void) = 0;
			
	};

	public __abstract __gc class CItemBase
	{
		public:
			// init/fin
			CItemBase(CFolderBase * pParent, String * sDisplayName);
			// properties
			__property CFolderBase * get_ParentFolder(void);
			__property String * get_DisplayName(void);
			__property virtual GregorView::Managed::ShellAttributes get_ShellAttributes(void) = 0;

		private:
			// fields
			CFolderBase * m_pParentFolder;
			String * m_pDisplayName;
			
	};

	public __abstract __gc class CFolderBase : public GregorView::Managed::CItemBase
	{
		public:
			// init/fin
			CFolderBase(String * sDisplayName);
			CFolderBase(CFolderBase * pParent, String * sDisplayName);
			// properties
			__property NFolderCollection * get_SubFolders(void);
			__property virtual bool get_HasSubFolders(void);
			__property GregorView::Managed::ShellAttributes get_ShellAttributes(void);
			// methods
			virtual IntPtr GetViewWindow(void) = 0;
			virtual void ReleaseViewWindow(IntPtr hWnd);
		
		protected:
			// methods
			virtual void ReadSubFolders(void) = 0;

		private:
			// fields
			NFolderCollection * m_pSubFolders;

	};

	public __gc class NFolderCollection : public CollectionBase
	{
		public:
			// init/fin
			NFolderCollection(CFolderBase * pParent);
			// properties
			__property CFolderBase * get_ParentFolder(void);
			__property CFolderBase * get_Element(int index);
			__property CFolderBase * get_Element(String * sDisplayName);
			// methods
			void Add(CFolderBase * pFolder);

		private:
			// fields
			CFolderBase * m_pParentFolder;

	};

};};
