// ####################################################################################
// Includes
// ####################################################################################

#include "stdafx.h"
#include "stdio.h"
#include "internal.h"

using namespace GregorView::Internal;


// ####################################################################################
// Functions
// ####################################################################################

const ITEMIDLIST * GregorView::Internal::Skip(const ITEMIDLIST * pcidl, const ITEMIDLIST * pcidlToSkip)
{
	const ITEMIDLIST * pcidlRet = NULL;
	if(pcidl != NULL && pcidlToSkip != NULL){
		size_t size = GetIDListSize(pcidlToSkip, false); // keep the terminator, so don't include it here
		// just chop off the NSE root ID list, but leave the terminator (in case pcidl refers to the NSE root folder)
		pcidlRet = (const ITEMIDLIST *) PTRADD(pcidl, size);
	}
	return pcidlRet;
} // SkipNseRoot

const ITEMIDLIST * GregorView::Internal::Next(const ITEMIDLIST * pcidl)
{
	const ITEMIDLIST * pcidlOut = NULL;
	if(pcidl == NULL){
		return NULL;
	}
	if(pcidl->mkid.cb != 0){
		// advance
		pcidlOut = (const ITEMIDLIST *) PTRADD(pcidl, pcidl->mkid.cb);
	}
	return pcidlOut;
}

const char * GregorView::Internal::GetIDListInfoString(const ITEMIDLIST * pcidl)
{
	static char sz[5000]; // static for speed && return
	sprintf(sz, "IDList address <0x%x> {", pcidl);
	// calculate levels, sum of pidl sizes
	int cLevels = 0;
	size_t size = 0;
	while(pcidl->mkid.cb != 0){
		// count up
		cLevels++;
		size += pcidl->mkid.cb;
		// info
		char szLevel[500];
		sprintf(szLevel, "level <%d> size <%d>, ", cLevels, pcidl->mkid.cb);
		strcat(sz, szLevel);
		// advance
		pcidl = (ITEMIDLIST*) PTRADD(pcidl, pcidl->mkid.cb);
	}
	// terminator
	size += TERM_SIZE;
	// end info
	char szEnd[100];
	sprintf(szEnd, "} cLevels <%d> totalSize: <%d>", cLevels, size);
	strcat(sz, szEnd);
	// OK
	return sz;
} // GetPidlInfoString

bool GregorView::Internal::IsTerminator(const ITEMIDLIST * pcidl)
{
	return (0 == pcidl->mkid.cb);
} // IsTerminator

const ITEMIDLIST * GregorView::Internal::GetNextItemID(const ITEMIDLIST * pcidl)
{
	if(IsTerminator(pcidl)){
		return NULL;
	}else{
		return (const ITEMIDLIST *) PTRADD(pcidl, pcidl->mkid.cb);
	}
} // GetNext

const ITEMIDLIST * GregorView::Internal::CloneIDList(const ITEMIDLIST * pcidl)
{
	const ITEMIDLIST * pcidlOut = NULL;
	// save start address for later use when copying
	const ITEMIDLIST * pcidlStart = pcidl;
	// calculate sum of pidl sizes
	int cLevels = 0;
	size_t size = 0;
	while(pcidl->mkid.cb != 0){
		cLevels++;
		size += pcidl->mkid.cb;
		pcidl = (ITEMIDLIST*) PTRADD(pcidl, pcidl->mkid.cb);
	}
	// terminator
	size += TERM_SIZE;
	// allocate
	void * p = GetAllocator().Allocate(size);
	if(p != NULL){
		// copy
		memcpy(p, pcidlStart, size);		
		// set return
		pcidlOut = reinterpret_cast<const ITEMIDLIST*>(p);
	}
	// info
	::GetTracer().TraceFormatted("::CloneIDList size <%d> cLevels <%d>", size, cLevels);
	// OK
	return pcidlOut;
} // CloneIDList

const ITEMIDLIST * GregorView::Internal::ConcatIDLists(const ITEMIDLIST * pcidl1, const ITEMIDLIST * pcidl2)
{
	// locals
	const ITEMIDLIST * pcidlRet = NULL;
	size_t size1 = GetIDListSize(pcidl1, false);
	size_t size2 = GetIDListSize(pcidl2, false);
	size_t sizeBoth = size1 + size2 + TERM_SIZE;
	// allocate
	void * p = GetAllocator().Allocate(sizeBoth);
	// concat
	if(p != NULL){
		// save start of new memory
		void * pStart = p;
		// copy both pidls + terminator
		memcpy(p, pcidl1, size1);
		p = PTRADD(p, size1);
		memcpy(p, pcidl2, size2);
		p = PTRADD(p, size2);
		memset(p, 0, TERM_SIZE);
		// OK
		pcidlRet = reinterpret_cast<const ITEMIDLIST *>(pStart);
	}
	// return
	return pcidlRet;	
} // ConcatIDLists

const ITEMIDLIST * GregorView::Internal::GetTerminator(void)
{
	static ITEMIDLIST idl;
	idl.mkid.cb = 0;
	return &idl;
} // GetTerminator

const size_t GregorView::Internal::GetIDListSize(const ITEMIDLIST * pcidl, bool fIncludeTerminator)
{
	size_t size = 0;
	// walk the pidl
	while(pcidl->mkid.cb != 0){
		size += pcidl->mkid.cb;
		pcidl = (ITEMIDLIST*) PTRADD(pcidl, pcidl->mkid.cb);
	}
	// include the terminator?
	if(fIncludeTerminator){
		size += TERM_SIZE;
	}
	// return
	return size;
} // GetIDListSize

void GregorView::Internal::ConcatPathOfNamesTo(const ITEMIDLIST * pcidl, char * psz, size_t bufferSize)
{
	while(false == ::IsTerminator(pcidl)){
		// all IDs are assumed to be GregorView IDs
		CGregorViewId * pID = AsGregorViewId(pcidl);
		pID->ConcatNameTo(psz, bufferSize);
		// next
		pcidl = GetNextItemID(pcidl);
	}
} // GetPathOfNames

CGregorViewId * GregorView::Internal::AsGregorViewId(const ITEMIDLIST * pcidl)
{
	if(pcidl->mkid.cb == sizeof(CGregorViewId)){
		const CGregorViewId * pID = reinterpret_cast<const CGregorViewId *>(pcidl);
		return const_cast<CGregorViewId*>(pID);
	}else{
		GetTracer().TraceFormatted("::AsGregorViewId wrongly tried to interpret %s", GetIDListInfoString(pcidl));
		return NULL;
	}
} // AsGregorViewId
